from torch.utils.data import Dataset
from pathlib import Path
import numpy as np
from torchvision import transforms
import csv
import h5py
from torchvision.transforms import ToTensor, Normalize, Resize
import torch
import temporal_transforms as ttfms

root = "..."

class SLRDataset(Dataset):
    def __init__(self, phase, img_size=None, dataset_name = None, completed = None,size=(112,112), evaluate = False):
        self.img_size = img_size
        filter_list = []
        class_list = set()
        with open('BosphorusSign22k.csv', "r", encoding="utf8") as f:
            for row in list(csv.reader(f))[1:]:
                subsetID = row[0]
                if 'General' in subsetID:
                   	filter_list.append(row[1]+'_'+row[4] +'_'+ row[5])  
                   	class_list.add(row[1])
        
        self.root = root
        if completed == "nocompleted":
            self.root = self.root + "nocompleted/"
        else:
            self.root = self.root + "completed/"
        
        if dataset_name is not None:
            self.path_dataset = Path(self.root+dataset_name)
            print(f"Dataset : {self.path_dataset}")
        
        with h5py.File(str(self.path_dataset),'r') as ds:
            keys_list = list(ds.keys())
            
        if phase == 'train':
            self.data_list = list(filter(lambda x: ('User_2' in x or 'User_5' in x or 'User_6' in x or 'User_7' in x or 'User_3' in x) , keys_list))
        elif phase == 'val':
            self.data_list = list(filter(lambda x: ('User_4' in x) , keys_list))
        elif phase == 'test':
            self.data_list = list(filter(lambda x: ('User_4' in x) , keys_list))
            
        self.class_count = len(class_list)
        self.class_list = class_list
   
        self.label2idx = {}
        for idx, data in enumerate(sorted(class_list)):
             self.label2idx[data]=idx

        self.frame_size = 32
        if phase == 'train':
            if evaluate:
                self.temporal_transform = ttfms.Uniform(self.frame_size,1)
            else:
                self.temporal_transform = ttfms.RandomUniform(self.frame_size,1)
        elif phase == 'val':
            self.temporal_transform = ttfms.Uniform(self.frame_size,1)
        elif phase == 'test':
            self.temporal_transform = ttfms.Uniform(self.frame_size,1)
        
        self.stfms = transforms.Compose([transforms.ToTensor(),
                                    transforms.Resize(size),
                                    transforms.Normalize(mean=[0.5], std=[0.5]),
                                  ])
        
        self.phase = phase
        
    def __len__(self):
        return len(self.data_list)

    def __getitem__(self, idx):

        path = self.data_list[idx]

        with h5py.File(str(self.path_dataset),'r') as ds:
            sample = ds[path][...]
            
        n_frames = sample.shape[0]

        frame_indices = self.temporal_transform(range(n_frames))

        label = int(self.data_list[idx].split('_')[0]) - 1 
        sample = sample[frame_indices]

        sample = torch.stack([self.stfms(img) for img in sample],0)
        sample = sample.permute((1, 0, 2, 3))

        return sample, label