from torch.utils.data import Dataset
from pathlib import Path
import numpy as np
from torchvision import transforms
import csv
import h5py
from torchvision.transforms import ToTensor, Normalize, Resize
import torch

root = "..."

class SLRDataset(Dataset):
    def __init__(self, phase, img_size=None, dataset_name = None, completed= None, evaluate = None, size=(112, 112)):
        self.img_size = img_size

        filter_list = []
        class_list = set()
        with open('BosphorusSign22k.csv', "r", encoding="utf8") as f:
            for row in list(csv.reader(f))[1:]:
                subsetID = row[0]
                if 'General' in subsetID:
                   	filter_list.append(row[1]+'_'+row[4] +'_'+ row[5])  
                   	class_list.add(row[1])

        self.root = root
        if completed == "nocompleted":
            self.root = self.root + "nocompleted/"
        else:
            self.root = self.root + "completed/"
            
        print(f"ROOT: {self.root}")        
        self.path_dataset = Path(self.root+dataset_name)
        with h5py.File(str(self.path_dataset),'r') as ds:
            keys_list = list(ds.keys())
            print(f"Dataset : {self.path_dataset}")
            
        if phase == 'train':
            self.data_list = list(filter(lambda x: ('User_2' in x or 'User_5' in x or 'User_6' in x or 'User_7' in x or 'User_3' in x) , keys_list))
        elif phase == 'val':
            self.data_list = list(filter(lambda x: ('User_4' in x) , keys_list))
        elif phase == 'test':
            self.data_list = list(filter(lambda x: ('User_4' in x) , keys_list))
                
        self.class_count = len(class_list)
        self.class_list = class_list
   
        self.label2idx = {}
        for idx, data in enumerate(sorted(class_list)):
             self.label2idx[data]=idx

        self.phase = phase
        
        self.tfms = transforms.Compose([transforms.ToTensor(),
                                        transforms.Resize((112, 112)),
                                        transforms.Normalize(mean=[0.5], std=[0.5]),
                                      ])
    def __len__(self):
        return len(self.data_list)

    def __getitem__(self, idx):

        with h5py.File(str(self.path_dataset),'r') as ds:
            sample = ds[self.data_list[idx]][...]
    
        label = int(self.data_list[idx].split('_')[0]) - 1 
 
        if len(sample.shape) == 3:
            if sample.shape[0] == 5:
                sample = torch.stack([self.tfms(img) for img in sample],0)
                sample = torch.squeeze(sample, dim=1)
            else:
                sample = self.tfms(sample)
        else:
            sample = self.tfms(sample)

        return sample, label
