from torch.utils.data import Dataset
from pathlib import Path
import numpy as np
from torchvision import transforms
import csv
import h5py
from torchvision.transforms import ToTensor, Normalize, Resize
import torch
import temporal_transforms as ttfms
import pickle

root = "..."

def readCSV(csv_name):
    path_list = []
    key_list = []
    with open(root+csv_name, "r", encoding="utf8") as f:
        for row in list(csv.reader(f)):
            path_list.append(row[0])
            pat = row[0]
            key_id = pat.split('|')[0].split('/')[0] + '_' +pat.split('|')[0].split('/')[1] + '_'+pat.split('|')[1]
            key_list.append(key_id)
    return key_list

def getLabel(key_id):
    return key_id.split(key_id.split('_')[4])[1][1:]

def loadpickle():
    path = 'label2idx_grk.pkl'
    with open(path, 'rb') as f:
        data = pickle.load(f)
    return data    


class SLRDataset(Dataset):
    def __init__(self, phase, img_size=None,dataset_name = None, ek = None, completed= None, size = None, evaluate = False):
        self.img_size = img_size

        class_list = set()

        if phase == 'train':
            csv_name = 'train_greek_iso.csv'
        elif phase == 'val':
            csv_name = "dev_greek_iso.csv"
        elif phase == 'test':
            csv_name = "test_greek_iso.csv"

        keys_list = readCSV(csv_name)
        self.data_list = keys_list
        label_list = []
        for key in keys_list:
            label_list.append(getLabel(key))

        label_list = set(label_list)
        label2idx = loadpickle()

        self.root = root
        if completed == "nocompleted":
            self.root = self.root + "nocompleted/"
        else:
            self.root = self.root + "completed/"
        
        self.label2idx = label2idx
        
        if dataset_name is not None:
            self.path_dataset = Path(self.root+dataset_name)
            print(f"Dataset : {self.path_dataset}")

        self.class_count = len(label2idx)
        
        self.class_list = class_list

        self.frame_size = 32
        if phase == 'train':
            if evaluate:
                self.temporal_transform = ttfms.Uniform(self.frame_size,1)
            else:
                self.temporal_transform = ttfms.RandomUniform(self.frame_size,1)
        elif phase == 'val':
            self.temporal_transform = ttfms.Uniform(self.frame_size,1)
        elif phase == 'test':
            self.temporal_transform = ttfms.Uniform(self.frame_size,1)
            
        self.stfms = transforms.Compose([transforms.ToTensor(),
                                    transforms.Resize(size),
                                    transforms.Normalize(mean=[0.5], std=[0.5]),
                                  ])
    
        self.phase = phase    
    
        
    def __len__(self):
        return len(self.data_list)

    def __getitem__(self, idx):

        with h5py.File(str(self.path_dataset),'r') as ds:
            sample = ds[self.data_list[idx]][...]
            
        path = self.data_list[idx]

        n_frames = sample.shape[0]

        frame_indices = self.temporal_transform(range(n_frames))

        label = getLabel(path)
        labelidx = self.label2idx[label]
        
        sample = sample[frame_indices]

        sample = torch.stack([self.stfms(img) for img in sample],0)

        sample = sample.permute((1, 0, 2, 3))

        return sample, int(labelidx)