from torch.utils.data import Dataset
from pathlib import Path
from torchvision import transforms
import csv
import h5py
from torchvision.transforms import ToTensor, Normalize, Resize
import torch
import temporal_transforms as ttfms
import pickle

root = "..."

def readCSV(csv_name):
    path_list = []
    key_list = []
    with open(root+csv_name, "r", encoding="utf8") as f:
        for row in list(csv.reader(f)):
            path_list.append(row[0])
            pat = row[0]
            key_id = pat.split('|')[0].split('/')[0] + '_' +pat.split('|')[0].split('/')[1] + '_'+pat.split('|')[1]
            key_list.append(key_id)
    return key_list


def loadpickle(path):
    with open(path, 'rb') as f:
        data = pickle.load(f)
    return data 

class SLRDataset(Dataset):
    def __init__(self, phase, img_size=None,dataset_name = None, ek = None, userid = '', completed = None,size=(112,112), evaluate = False, experiment = None):
        self.img_size = img_size
        
        
        self.root = root
        if completed == "nocompleted":
            self.root = self.root + "nocompleted/"
        else:
            self.root = self.root + "completed/"
        
        if dataset_name is not None:
            self.path_dataset = Path(self.root+dataset_name)
            print(f"Dataset : {self.path_dataset}")

        with h5py.File(str(self.path_dataset),'r') as ds:
            keys_list = list(ds.keys())

        class_list = set()
        
        if experiment == "e1":
            if phase == 'train':
                self.data_list = list(filter(lambda x: ('001' == x.split('_')[1] or '002' == x.split('_')[1] or '003' == x.split('_')[1] or '004' == x.split('_')[1] or '006' == x.split('_')[1] or '007' == x.split('_')[1] or '008' == x.split('_')[1] or '009' == x.split('_')[1]) , keys_list))
            elif phase == 'val':
                self.data_list = list(filter(lambda x: ('005' == x.split('_')[1] or '010' == x.split('_')[1]) , keys_list))
            elif phase == 'test':
                self.data_list = list(filter(lambda x: ('005' == x.split('_')[1] or '010' == x.split('_')[1]) , keys_list))
        elif experiment == "e2":
            if phase == 'train':
                self.data_list = list(filter(lambda x: (userid != x.split('_')[1]) , keys_list))
            elif phase == 'val':
                self.data_list = list(filter(lambda x: (userid == x.split('_')[1]) , keys_list))
            elif phase == 'test':
                self.data_list = list(filter(lambda x: (userid == x.split('_')[1]) , keys_list))
        elif experiment == "e3":
            X_train = loadpickle("train_lsa64.pkl")
            X_test = loadpickle("test_lsa64.pkl")
            
            if phase == 'train':
                self.data_list = X_train
            elif phase == 'val':
                self.data_list = X_test
            elif phase == 'test':
                self.data_list = X_test

        self.class_count = len(class_list)
        self.class_list = class_list

        self.label2idx = {}
        for idx, data in enumerate(sorted(class_list)):
             self.label2idx[data]=idx

        self.frame_size = 32
        if phase == 'train':
            if evaluate:
                self.temporal_transform = ttfms.Uniform(self.frame_size,1)
            else:
                self.temporal_transform = ttfms.RandomUniform(self.frame_size,1)
        elif phase == 'val':
            self.temporal_transform = ttfms.Uniform(self.frame_size,1)
        elif phase == 'test':
            self.temporal_transform = ttfms.Uniform(self.frame_size,1)
        
        self.stfms = transforms.Compose([transforms.ToTensor(),
                                    transforms.Resize(size),
                                    transforms.Normalize(mean=[0.5], std=[0.5]),
                                  ])
        
        self.phase = phase
    
    def __len__(self):
        return len(self.data_list)

    def __getitem__(self, idx):
        
        with h5py.File(str(self.path_dataset),'r') as ds:
            sample = ds[self.data_list[idx]][...]
            
        label = int(self.data_list[idx].split('_')[0])-1
        n_frames = sample.shape[0]
        frame_indices = self.temporal_transform(range(n_frames))
        sample = sample[frame_indices]

        sample = torch.stack([self.stfms(img) for img in sample],0)
        sample = torch.squeeze(sample, dim=1)

        return sample, int(label)
        
    