## Standard libraries
import os
import numpy as np
from learner import fit
import time
from modules import ClassificationModule
from torch.utils.data import DataLoader
import pytorch_lightning as pl
import torch
from pytorch_lightning.callbacks import LearningRateMonitor, ModelCheckpoint, TQDMProgressBar
from pytorch_lightning.loggers import TensorBoardLogger
from dataset_Bosph_mhi import SLRDataset as ds_mhi
from dataset_Bosph import SLRDataset as ds_rgb
from tqdm import tqdm

def test_SVM(train, train_label, test, test_label):
    from sklearn.svm import SVC
    logisticRegr = SVC(random_state=60)
    logisticRegr.fit(train, train_label)
    scr = logisticRegr.score(test, test_label)
    return scr

def test(model,ds):
    model.cuda()
    model.eval()
    
    feature_list = []
    label_list = []
    
    for idx, (data, label) in enumerate(tqdm(ds)):
        with torch.no_grad():
            res = model.model.conv1(torch.unsqueeze(data, 0).cuda())
            res = model.model.model(res.cuda()).cpu()
            feature_list.append(res.reshape(-1).numpy())
        label_list.append(label)
        
    feature_list = np.asarray(feature_list,dtype=np.float32)
        
    return feature_list,label_list

device = torch.device("cuda:0") if torch.cuda.is_available() else torch.device("cpu")

def get_feature(save_name, modelName = 'resnet18v6', channel = None, dataModule = None, data_name = None):
    data_name = data_name + ".hdf5"
    ds_train = dataModule(phase='train',dataset_name=data_name)
    ds_val = dataModule(phase='val', dataset_name=data_name)
    ds_test = dataModule(phase='test', dataset_name=data_name)
    
    module_setting = {
                      'model_name' : modelName,
                      'model_hparams':{"num_classes": 174,
                                       "pretrained" : True,
                                        "first_filter_size":1,
                                     },
                      'optimizer_name':"SGD",
                      'optimizer_hparams':{"lr": 1e-3,
                                            "momentum":0.9,
                                          },
                      'lr_scheduler_hparams':{
                          'step_size': 10
                      }
    }
    
    model = ClassificationModule(**module_setting)
    model = model.load_from_checkpoint(save_name)
    
    X_train, y_train = test(model, ds_train)
    X_test, y_test = test(model, ds_val)
    
    scr = test_SVM(X_train, y_train, X_test, y_test)
    
    return X_train, y_train, X_test, y_test, scr


def fusion(all_data):
    train_data = []
    test_data = []
    train_label = []
    test_label = []
    
    for X_train, y_train, X_test, y_test, scr in all_data:
        train_data.append(X_train)
        test_data.append(X_test)
        train_label = y_train
        test_label = y_test
        print(scr)
    
    train_data = np.concatenate(train_data, axis=1)
    test_data = np.concatenate(test_data, axis=1)
    
    return train_data, train_label, test_data, test_label

all_data = []

def get_final_features(save_name):
    modelName = save_name.split('/')[2].split('_')[0]
    print(modelName)
    dataName = '_'.join(save_name.split('/')[2].split('_bs')[0].split('_')[1:])
    print(dataName)
    if 'mhi' in save_name.split('/')[1]:
        ds = ds_mhi
        print("**")
    else:
        ds = ds_rgb
    
    tempData = get_feature(save_name,  modelName = modelName,  dataModule = ds, data_name= dataName)
    return tempData


save_name = 'file.ckpt'

modelName = save_name.split('/')[2].split('_')[0]
print(modelName)
dataName = '_'.join(save_name.split('/')[2].split('_bs')[0].split('_')[1:])
print(dataName)
ds = ds_rgb

data_name = dataName + ".hdf5"
ds_train = ds(phase='train',dataset_name=data_name,size=(56,112))
ds_val = ds(phase='val', dataset_name=data_name,size=(56,112))
ds_test = ds(phase='test', dataset_name=data_name,size=(56,112))

module_setting = {
                  'model_name' : modelName,
                  'model_hparams':{"num_classes": 174,
                                   "pretrained" : True,
                                    "first_filter_size":1,

                                 },
                  'optimizer_name':"SGD",
                  'optimizer_hparams':{"lr": 1e-3,
                                        "momentum":0.9,
                                      },
                  'lr_scheduler_hparams':{
                      'step_size': 10
                  }
}

model = ClassificationModule(**module_setting)
model = model.load_from_checkpoint(save_name)

model.eval()

filter_list = []
class_list = set()
label2word = {}
import csv
with open('BosphorusSign22k.csv', "r", encoding="utf8") as f:
    for row in list(csv.reader(f))[1:]:
        subsetID = row[0]
        if 'General' in subsetID:
            filter_list.append(row[1]+'_'+row[4] +'_'+ row[5])  
            class_list.add(row[1])
               
            label2word[row[1]] = row[2]

import pickle
with open('label2idx.pkl', 'rb') as f:
    label2idx = pickle.load(f)
    

idx2label = {}
for k,v in label2idx.items():
    idx2label[int(v)-1] = k
    
for d,l in ds_test:
    d = torch.unsqueeze(d,0)
    with torch.no_grad():
        s = model(d)
        tahmin = s.argmax().item()
        if s.argmax() != l:
            print(label2word[idx2label[tahmin]],label2word[idx2label[l]])

train_data, train_label, test_data, test_label = fusion(all_data)
test_SVM(train_data, train_label, test_data, test_label)