import torch.utils.data as data
import pytorch_lightning as pl
from train import train_model


task_list = ['classification','segmentation']

default_classification_module_setting = {
                  'model_name' : 'resnet18',
                  'model_hparams':{"num_classes": 10,
                                   "pretrained" : True,
                                 },
                  'optimizer_name':"SGD",
                  'optimizer_hparams':{"lr": 1e-3,
                                       "momentum":0.9
                                      }
}

default_classification_train_setting = {
    'max_epochs':30,
    'gpus' : 1,
    'monitor' :'val_acc',
    'evaluate_metric' :'test_acc',
    'bs':64
}


def fit(module, save_name, 
        train_dataset = None,val_dataset = None,test_dataset = None, 
        pretrained_model = False, checkpoint_path = None, transfer_learning = False, module_setting = None, train_setting = None):
    
    pl.seed_everything(42)

    
    bs = train_setting['bs']
    num_workers =  train_setting['num_workers']
    
    train_loader = data.DataLoader(train_dataset, batch_size=bs, shuffle=True, drop_last=True, num_workers=num_workers)
    
    if val_dataset is not None:
        val_loader = data.DataLoader(val_dataset, batch_size=bs, shuffle=False, drop_last=False, num_workers=num_workers)
    else: val_loader = None
        
    if test_dataset is not None:
        test_loader = data.DataLoader(test_dataset, batch_size=bs, shuffle=False, drop_last=False, num_workers=num_workers)
    else: test_loader = None
    


    my_module = module(**module_setting)
    
    print(my_module)

    model, results = train_model(module = my_module, 
                                 train_loader=train_loader,
                                 val_loader=val_loader,
                                 test_loader=test_loader,
                                 save_name = save_name,
                                 pretrained_model = pretrained_model,
                                 checkpoint_path = checkpoint_path,
                                 transfer_learning=transfer_learning,
                                 **train_setting
                                 )
    
    return results
