import cv2
import mediapipe as mp
import pickle
import glob

mp_hands = mp.solutions.holistic

def write_pickle(file_name, data):
    try:
        with open(file_name, 'rb') as f:
            veri = pickle.load(f)
        with open(file_name, 'wb') as f:
            veri.update(data)
            pickle.dump(veri, f)
    except Exception:
        with open(file_name, 'wb') as f:
            veri = data
            pickle.dump(veri, f)
            
            
def get_landmark(path_list):
    cap = cv2.VideoCapture(path_list)
    
    landmark_data = []

    with mp_hands.Holistic(
        static_image_mode=False,
        smooth_landmarks=True,
        enable_segmentation=False,
        refine_face_landmarks=True,
        min_detection_confidence=0.5,
        min_tracking_confidence=0.5) as hands:
        
      while cap.isOpened():
        success, image = cap.read()
        
        if not success:
            break
        
        image.flags.writeable = False
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        results = hands.process(image)
        landmark_data.append(results)
        
    return landmark_data


path_videos = ".../path_video_dataset.../"
list_path_video = sorted(glob.glob(path_videos+"*.mp4"))

path_pose_data = ".../path_pose_data.../"


for idx, pat in enumerate(list_path_video):
    key_id = pat.split('/')[-1].split('.')[0]
    
    lm = get_landmark(pat)

    lm_pose      = [output.pose_landmarks for output in lm]
    lm_face      = [output.face_landmarks for output in lm]
    lm_lefthand  = [output.left_hand_landmarks for output in lm]
    lm_righthand = [output.right_hand_landmarks for output in lm]
    lm_poseworld = [output.pose_world_landmarks for output in lm]
    
    data = {'pose':lm_pose, 'face':lm_face,'lhands':lm_lefthand,'rhands':lm_righthand,'pose_world':lm_poseworld}
    
    pdata = path_pose_data + key_id + ".pkl"
    write_pickle(pdata, data)
        
    print(f"{idx+1}/{len(list_path_video)}")

