import drawing_styles_custom as mp_drawing_styles
import numpy as np
import mediapipe as mp
import connections as con
import cv2
from utils import getTemplateImage, readPickle
from utils import array2landmark_pose, array2landmark_hands, array2landmark_face
from utils import landmark2array_pose, refinement
from utils import get_mhi, scale_pose, crop_hands, arangement_hands_data, toGray, get_mhi_multichannel
from utils import landmark2array_pose, landmark2array_hands, landmark2array_face
from utils import resize_list, get_mhi_finger
import utils_drawing as ud

mp_drawing = mp.solutions.drawing_utils
mp_holistic = mp.solutions.holistic

def extract(path, vpath):
    
    frames = []
    cap = cv2.VideoCapture(vpath)
    while True:
        ret, frame = cap.read()
        if not ret:
            break
        
        frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB) 

        frames.append(frame)
    frames = np.asarray(frames)
    
    path_holistic = path
    data = readPickle(path_holistic)

    data['lhands'] = refinement(data['lhands'], landmark2array_hands, array2landmark_hands)
    data['rhands'] = refinement(data['rhands'], landmark2array_hands, array2landmark_hands)
    data['body']   = refinement(data['body'], landmark2array_pose, array2landmark_pose)
    data['face']   = refinement(data['face'], landmark2array_face, array2landmark_face)
    
    mask_list_body = []
    mask_list_hands = []
    mask_list_face = []
    
    mask_THUMB = []
    mask_INDEX = []
    mask_MIDDLE = []
    mask_RING = []
    mask_PINKY = []  
    mask_list_finger = []
    
    Rhand_list = []
    Lhand_list = []
    
    RGB_Rhand_list = []
    RGB_Lhand_list = []
    
    for pidx, (landmark_pose, landmark_face, landmark_lhands, landmark_rhands, frame) in enumerate(zip(
            data['body'], 
            data['face'], 
            data['lhands'], 
            data['rhands'],
            frames)):
                
        mask_pose = getTemplateImage(size=(1080,1920,3))
        mask_face = getTemplateImage(size=(1080,1920,3))
        
        mask_finger = getTemplateImage(size=(270,480,3))
        mask_fingers = [mask_finger.copy(),mask_finger.copy(),mask_finger.copy(),mask_finger.copy(),mask_finger.copy()]
        
        mask_hands_L = getTemplateImage(size=(1080,1920,3))
        mask_hands_R = getTemplateImage(size=(1080,1920,3))
        
        mask_hands = getTemplateImage(size=(1080,1920,3))
        
        ud.draw_landmarks(
            mask_pose,
            landmark_pose,
            con.POSE_CONNECTIONS,
            None,
            mp_drawing_styles.get_default_pose_connections_style2(),
            'body')
        
        ud.draw_landmarks(
            mask_pose,
            landmark_face,
            mp_holistic.FACEMESH_CONTOURS,
            None,
            mp_drawing_styles.get_default_face_mesh_contours_style(),
            'face')
        
        ud.draw_landmarks(
            mask_face,
            landmark_face,
            mp_holistic.FACEMESH_CONTOURS,
            None,
            mp_drawing_styles.get_default_face_mesh_contours_style(),
            'face')
        
        if data['lhands'] is not None:
            for id_mask in range(len(mask_fingers)):
                mp_drawing.draw_landmarks(
                    mask_fingers[id_mask],
                    landmark_lhands,
                    con.HAND_CONNECTIONS_ARRAY[id_mask],
                    None,
                    mp_drawing_styles.get_default_hand_connections_style_idx(id_mask))
                
            ud.draw_landmarks(
                mask_hands_L,
                landmark_lhands,
                con.HAND_CONNECTIONS_FULL,
                None,
                mp_drawing_styles.get_default_hand_connections_style(),
                'hands')
            
            ud.draw_landmarks(
                mask_pose,
                landmark_lhands,
                con.HAND_CONNECTIONS_FULL,
                None,
                mp_drawing_styles.get_default_hand_connections_style(),
                'hands')
            
            ud.draw_landmarks(
                mask_hands,
                landmark_lhands,
                con.HAND_CONNECTIONS_FULL,
                None,
                mp_drawing_styles.get_default_hand_connections_style(),
                'hands')
            
        Lhand_list.append(toGray(crop_hands(landmark_lhands, mask_hands_L)))
        RGB_Lhand_list.append(crop_hands(landmark_lhands, mask_hands_L))

        if data['rhands'] is not None:
            for id_mask in range(len(mask_fingers)):
                mp_drawing.draw_landmarks(
                    mask_fingers[id_mask],
                    landmark_rhands,
                    con.HAND_CONNECTIONS_ARRAY[id_mask],
                    None,
                    mp_drawing_styles.get_default_hand_connections_style_idx(id_mask))
    
            ud.draw_landmarks(
                mask_hands_R,
                landmark_rhands,
                con.HAND_CONNECTIONS_FULL,
                None,
                mp_drawing_styles.get_default_hand_connections_style(),
                'hands')
            
            ud.draw_landmarks(
                mask_pose,
                landmark_rhands,
                con.HAND_CONNECTIONS_FULL,
                None,
                mp_drawing_styles.get_default_hand_connections_style(),
                'hands')
            
            ud.draw_landmarks(
                mask_hands,
                landmark_rhands,
                con.HAND_CONNECTIONS_FULL,
                None,
                mp_drawing_styles.get_default_hand_connections_style(),
                'hands')            
            
        Rhand_list.append(toGray(crop_hands(landmark_rhands, mask_hands_R)))
        RGB_Rhand_list.append(crop_hands(landmark_rhands, mask_hands_R))


        mask_THUMB.append(toGray(mask_fingers[0]))
        mask_INDEX.append(toGray(mask_fingers[1]))
        mask_MIDDLE.append(toGray(mask_fingers[2]))
        mask_RING.append(toGray(mask_fingers[3]))
        mask_PINKY.append(toGray(mask_fingers[4]))
        
        mask_list_body.append(toGray(mask_pose))
        mask_list_hands.append(toGray(mask_hands))
        mask_list_face.append(toGray(mask_face))
        
    
    mask_list_finger = np.asarray([mask_THUMB,
                                    mask_INDEX,
                                    mask_MIDDLE,
                                    mask_RING,
                                    mask_PINKY])
    
    mask_list_body = np.asarray(mask_list_body)
    mask_list_hands = np.asarray(mask_list_hands)
    mask_list_face = np.asarray(mask_list_face)
    
    scale_hands = scale_pose(mask_list_hands,[data['lhands'], data['rhands']],tag_list=['lhands','rhands'])
    scale_body = scale_pose(mask_list_body, [data['body'],data['lhands'], data['rhands'],data['face']],tag_list=['body','lhands','rhands','face'])
    scale_face = scale_pose(mask_list_face, [data['face']],tag_list=['face'])  

    Rhand_list = arangement_hands_data(Rhand_list)
    Lhand_list = arangement_hands_data(Lhand_list)
    mergedhands = np.concatenate([Rhand_list,Lhand_list],axis=2)  
    
    body = mask_list_body[:,scale_body[0]:scale_body[1],scale_body[2]:scale_body[3]]
    frames = frames[:,scale_body[0]:scale_body[1],scale_body[2]:scale_body[3]]  

    hands = mask_list_hands[:,scale_hands[0]:scale_hands[1],scale_hands[2]:scale_hands[3]]  
    face = mask_list_face[:,scale_face[0]:scale_face[1],scale_face[2]:scale_face[3]]  
    finger = mask_list_finger[:,:,scale_hands[0]//4:scale_hands[1]//4,scale_hands[2]//4:scale_hands[3]//4]

    im_size = (112,112)
    mhi_body_1 = get_mhi(body,im_size=im_size)
    mhi_body_3 = get_mhi_multichannel(body,3,im_size=im_size)                                   
    
    mhi_face_1 = get_mhi(face,im_size=im_size)
    mhi_face_3 = get_mhi_multichannel(face,3,im_size=im_size)   

    mhi_raw_1 = get_mhi(frames,im_size=im_size)
    mhi_raw_3 = get_mhi_multichannel(frames,3,im_size=im_size)                           
    
    mhi_finger = get_mhi_finger(finger, im_size=im_size)
    
    mhi_hands_1 = get_mhi(hands,im_size=im_size)
    mhi_hands_3 = get_mhi_multichannel(hands,3,im_size=im_size)
    
    face = resize_list(face)
    body = resize_list(body)
    hands = resize_list(hands)
    
    return {
            'body':body, 
            'mhi_body_1': mhi_body_1,
            'mhi_body_3': mhi_body_3,
            'face':face, 
            'mhi_face_1': mhi_face_1,
            'mhi_face_3': mhi_face_3,
            'hands':hands,
            'mhi_hands_1':mhi_hands_1,
            'mhi_hands_3':mhi_hands_3,
            'mergedhands':mergedhands,
            'raw':frames,
            'mhi_raw_1':mhi_raw_1,
            'mhi_raw_3':mhi_raw_3,
            'mhi_finger':mhi_finger,
            }
