import numpy as np
import cv2

class MHI:
    def __init__(self, MHI_DURATION = 10, DEFAULT_THRESHOLD = 32):
        self.MHI_DURATION = MHI_DURATION
        self.DEFAULT_THRESHOLD = DEFAULT_THRESHOLD
        
    def extract_feature(self, frame_array):
        h, w = frame_array[0].shape[:2]
        prev_frame = frame_array[0].copy()
        motion_history = np.zeros((h, w), np.float32)
        timestamp = 0
        
        for frame in frame_array:
            frame_diff = cv2.absdiff(frame, prev_frame)
            gray_diff = cv2.cvtColor(frame_diff, cv2.COLOR_BGR2GRAY)
            ret, fgmask = cv2.threshold(gray_diff, self.DEFAULT_THRESHOLD, 1, cv2.THRESH_BINARY)
            timestamp += 1

            cv2.motempl.updateMotionHistory(fgmask, motion_history, timestamp, self.MHI_DURATION)
            mh = np.uint8(np.clip((motion_history-(timestamp-self.MHI_DURATION)) / self.MHI_DURATION, 0, 1)*255)
            prev_frame = frame.copy()
            
        return mh
    
    def extract_feature_gray(self, frame_array):
        h, w = frame_array[0].shape[:2]
        prev_frame = frame_array[0].copy()
        motion_history = np.zeros((h, w), np.float32)
        timestamp = 0
        
        for frame in frame_array:
            frame_diff = cv2.absdiff(frame, prev_frame)
            gray_diff = frame_diff
            ret, fgmask = cv2.threshold(gray_diff, self.DEFAULT_THRESHOLD, 1, cv2.THRESH_BINARY)
            timestamp += 1

            cv2.motempl.updateMotionHistory(fgmask, motion_history, timestamp, self.MHI_DURATION)
            
            mh = np.uint8(np.clip((motion_history-(timestamp-self.MHI_DURATION)) / self.MHI_DURATION, 0, 1)*255)
            prev_frame = frame.copy()
            
        return mh
        
    def show_video(self, video_src = None):
        assert not video_src is None, "video not found!"
        cam = cv2.VideoCapture(video_src)
        ret, frame = cam.read()
        h, w = frame.shape[:2]
        prev_frame = frame.copy()
        motion_history = np.zeros((h, w), np.float32)
        timestamp = 0
        
        while True:
            ret, frame = cam.read()
            if not ret:
                break
            frame_diff = cv2.absdiff(frame, prev_frame)
            gray_diff = cv2.cvtColor(frame_diff, cv2.COLOR_BGR2GRAY)
            ret, fgmask = cv2.threshold(gray_diff, self.DEFAULT_THRESHOLD, 1, cv2.THRESH_BINARY)
            timestamp += 1

            cv2.motempl.updateMotionHistory(fgmask, motion_history, timestamp, self.MHI_DURATION)
            
            mh = np.uint8(np.clip((motion_history-(timestamp-self.MHI_DURATION)) / self.MHI_DURATION, 0, 1)*255)
            cv2.imshow('motempl', mh)
            cv2.imshow('raw', frame)
            
            prev_frame = frame.copy()
            if 0xFF & cv2.waitKey(5) == 27:
                break
        cv2.destroyAllWindows()
     
